@@ -2,8 +2,8 @@
 
 let projectName = "unison"
 let majorVersion = 2
-let minorVersion = 12
-let pointVersionOrigin = 14 (* Revision that corresponds to point version 0 *)
+let minorVersion = 13
+let pointVersionOrigin = 59 (* Revision that corresponds to point version 0 *)
 
 (* You shouldn't need to edit below. *)
 
@@ -34,15 +34,15 @@ let pointVersionOrigin = 14 (* Revision that corresponds to point version 0 *)
 
    The first two lines are obvious.  The last line says that Subversion
    revision 28 corresponds to a 0 point release.  Since we were at revision
-   27 before the commit, then we will be at 28 after the commit, so after
-   the commit we will be at Unison version 3.0.0.
-
+   27 and we're going to do a commit before making a release, we
+   will be at 28 after the commit and this will be Unison version 3.0.0.
 *)
 
-let revisionString = "$Rev: 14 $";;
+let revisionString = "$Rev: 75 $";;
 let revision = Scanf.sscanf revisionString "$Rev: %d " (fun x -> x);;
 let pointVersion = revision - pointVersionOrigin;;
 
+Printf.printf "MAJORVERSION=%d.%d\n" majorVersion minorVersion;;
 Printf.printf "VERSION=%d.%d.%d\n" majorVersion minorVersion pointVersion;;
 Printf.printf "NAME=%s\n" projectName;;
 
@@ -51,9 +51,3 @@ Printf.printf "NAME=%s\n" projectName;;
 
 
 
-
-
-
-
-
-
