@@ -6,11 +6,6 @@ external isMacOSXPred : unit -> bool = "isMacOSX"
 
 let isMacOSX = isMacOSXPred ()
 
-(* FIX: this should be somewhere else *)
-(* Only used to check whether pty is supported *)
-external isLinuxPred : unit -> bool = "isLinux"
-let isLinux = isLinuxPred ()
-
 (****)
 
 let rsrcSync =
@@ -136,7 +131,7 @@ let openDouble fspath path =
         entries := (id, (ofs, len)) :: !entries
       done;
       (path, inch, !entries)))
-    (fun () -> close_in inch)
+    (fun () -> close_in_noerr inch)
 
 (****)
 
@@ -219,20 +214,21 @@ let getFileInfos fspath path typ =
             let (fspath, path) = Fspath.findWorkingDir fspath path in
             let (doublePath, inch, entries) = openDouble fspath path in
             let (rsrcOffset, rsrcLength) =
-              try List.assoc `RSRC entries with Not_found ->
+              try Safelist.assoc `RSRC entries with Not_found ->
                 (0L, 0L)
             in
             let finfo =
               protect (fun () ->
                 try
-                  let (ofs, len) = List.assoc `FINFO entries in
+                  let (ofs, len) = Safelist.assoc `FINFO entries in
                   if len <> finfoLength then fail doublePath "bad finder info";
-                  readDoubleFromOffset doublePath inch ofs 32
+                  let res = readDoubleFromOffset doublePath inch ofs 32 in
+                  close_in inch;
+                  res
                 with Not_found ->
                   "")
-                (fun () -> close_in inch)
+                (fun () -> close_in_noerr inch)
             in
-            close_in inch;
             let stats = Unix.LargeFile.stat doublePath in
             { ressInfo =
                 if rsrcLength = 0L then NoRess else
@@ -282,25 +278,27 @@ let setFileInfos fspath path finfo =
       begin try
         let (doublePath, inch, entries) = openDouble fspath path in
         begin try
-          let (ofs, len) = List.assoc `FINFO entries in
+          let (ofs, len) = Safelist.assoc `FINFO entries in
           if len <> finfoLength then fail doublePath "bad finder info";
           let fullFinfo =
             protect
-              (fun () -> readDoubleFromOffset doublePath inch ofs 32)
-              (fun () -> close_in inch)
+              (fun () ->
+                let res = readDoubleFromOffset doublePath inch ofs 32 in
+                close_in inch;
+                res)
+              (fun () -> close_in_noerr inch)
           in
-          close_in inch;
           let outch =
             open_out_gen [Open_wronly; Open_binary] 0o600 doublePath in
           protect
             (fun () ->
                writeDoubleFromOffset doublePath outch ofs
-                 (insertInfo fullFinfo finfo))
+                 (insertInfo fullFinfo finfo);
+               close_out outch)
             (fun () ->
-               close_out outch);
-          close_out outch
+               close_out_noerr outch);
         with Not_found ->
-          close_in inch;
+          close_in_noerr inch;
           raise (Util.Transient
                    (Format.sprintf
                       "Unable to set the file type and creator: \n\
@@ -325,7 +323,7 @@ let setFileInfos fspath path finfo =
             output_string outch "\000\000\000\032"; (* length *)
             output_string outch (insertInfo (emptyFinderInfo ()) finfo);
             close_out outch)
-            (fun () -> close_out outch)
+            (fun () -> close_out_noerr outch)
         end
       end)
 
@@ -384,24 +382,25 @@ let ressDummy = NoRess
 (****)
 
 let openRessIn fspath path =
-  Util.convertUnixErrorsToTransient "reading ressource fork" (fun () ->
+  Util.convertUnixErrorsToTransient "reading resource fork" (fun () ->
     try
-      open_in_gen
-        [Open_rdonly; Open_binary] 0o444
-        (Fspath.concatToString fspath (ressPath path))
+      Unix.in_channel_of_descr
+        (Unix.openfile
+           (Fspath.concatToString fspath (ressPath path))
+           [Unix.O_RDONLY] 0o444)
     with Unix.Unix_error (Unix.ENOTDIR, _, _) ->
       let (doublePath, inch, entries) = openDouble fspath path in
       try
-        let (rsrcOffset, rsrcLength) = List.assoc `RSRC entries in
+        let (rsrcOffset, rsrcLength) = Safelist.assoc `RSRC entries in
         protect (fun () -> LargeFile.seek_in inch rsrcOffset)
-          (fun () -> close_in inch);
+          (fun () -> close_in_noerr inch);
         inch
       with Not_found ->
-        close_in inch;
-        raise (Util.Transient "No ressource fork found"))
+        close_in_noerr inch;
+        raise (Util.Transient "No resource fork found"))
 
 let openRessOut fspath path length =
-  Util.convertUnixErrorsToTransient "writing ressource fork" (fun () ->
+  Util.convertUnixErrorsToTransient "writing resource fork" (fun () ->
     try
       Unix.out_channel_of_descr
         (Unix.openfile
@@ -421,11 +420,11 @@ let openRessOut fspath path length =
         output_string outch "\000\000\000\009"; (* Finder info *)
         output_string outch "\000\000\000\050"; (* offset *)
         output_string outch "\000\000\000\032"; (* length *)
-        output_string outch "\000\000\000\002"; (* Ressource fork *)
+        output_string outch "\000\000\000\002"; (* Resource fork *)
         output_string outch "\000\000\000\082"; (* offset *)
         output_string outch (setInt4 (Uutil.Filesize.toInt64 length));
                                                 (* length *)
         output_string outch (emptyFinderInfo ());
         flush outch)
-        (fun () -> close_out outch);
+        (fun () -> close_out_noerr outch);
       outch)
