@@ -72,29 +72,33 @@ let _ = Util.debugPrinter := Some(debug)
 let logging =
   Prefs.createBool "log" true
     "record actions in file specified by logfile preference"
-    "When this flag is set, Unison will log all changes to the filesystems 
-     on a file." 
-    
+    "When this flag is set, Unison will log all changes to the filesystems
+     on a file."
+
 let logfile =
   Prefs.createString "logfile"
     (Util.fileInHomeDir "unison.log")
     "Log file name"
-    "By default, logging messages will be appended to the file 
-     \\verb|unison.log| in your HOME directory.  Set this preference if 
+    "By default, logging messages will be appended to the file
+     \\verb|unison.log| in your HOME directory.  Set this preference if
      you prefer another file."
-    
+
 let logch = ref None
 
-let getLogch() =
+let rec getLogch() =
   Util.convertUnixErrorsToFatal "getLogch" (fun() ->
   match !logch with
     None ->
-      let ch = open_out_gen 
-                 [Open_wronly; Open_append; Open_creat] 0o600
-                 (Prefs.read logfile) in
-      logch := Some(ch);
-      ch 
-  | Some(ch) -> ch)
+      let file = Prefs.read logfile in
+      let ch =
+        open_out_gen [Open_wronly; Open_append; Open_creat] 0o600 file in
+      logch := Some (ch, file);
+      ch
+  | Some(ch, file) ->
+      if Prefs.read logfile = file then ch else begin
+        close_out ch;
+        logch := None; getLogch ()
+      end)
 
 let sendLogMsgsToStderr = ref true
 
