@@ -206,6 +206,14 @@ let ignoreTransientErrors thunk =
   with
     Transient(s) -> ()
 
+let printException e =
+  try
+    raise e
+  with
+    Transient s -> s
+  | Fatal s -> s
+  | e -> Printexc.to_string e
+
 (* Safe version of Unix getenv -- raises a comprehensible error message if
    called with an env variable that doesn't exist                            *)
 let safeGetenv var =
@@ -244,14 +252,28 @@ let time () =
   convertUnixErrorsToTransient "time" Unix.time
 
 let time2string timef =
-  let time = localtime timef in
-  Printf.sprintf
-    "%2d:%.2d on %2d %3s, %4d"
-    time.Unix.tm_hour
-    time.Unix.tm_min
-    time.Unix.tm_mday
-    (monthname time.Unix.tm_mon)
-    (time.Unix.tm_year + 1900)
+  try
+    let time = localtime timef in
+(* Old-style:
+    Printf.sprintf
+      "%2d:%.2d:%.2d on %2d %3s, %4d"
+      time.Unix.tm_hour
+      time.Unix.tm_min
+      time.Unix.tm_sec
+      time.Unix.tm_mday
+      (monthname time.Unix.tm_mon)
+      (time.Unix.tm_year + 1900)
+*)
+    Printf.sprintf
+      "%4d-%02d-%02d at %2d:%.2d:%.2d"
+      (time.Unix.tm_year + 1900)
+      (time.Unix.tm_mon + 1)
+      time.Unix.tm_mday
+      time.Unix.tm_hour
+      time.Unix.tm_min
+      time.Unix.tm_sec
+  with Transient _ ->
+    "(invalid date)"
 
 let percentageOfTotal current total =
   (int_of_float ((float current) *. 100.0 /. (float total)))
@@ -364,12 +386,12 @@ let fileInHomeDir n =
     Filename.concat (safeGetenv "HOME") n
   else if osType = `Win32 then
     let dirString =
-      try Unix.getenv "UNISON" (* Use UNISON dir if it is set *)
-      with Not_found ->
       try Unix.getenv "HOME" (* Windows 9x with Cygwin HOME set *)
       with Not_found ->
       try Unix.getenv "USERPROFILE" (* Windows NT/2K standard *)
       with Not_found ->
+      try Unix.getenv "UNISON" (* Use UNISON dir if it is set *)
+      with Not_found ->
       "c:/" (* Default *) in
     Filename.concat dirString n
   else
